Type: Package
Package: packrat
Title: A Dependency Management System for Projects and their R Package
        Dependencies
Version: 0.9.3
Authors@R: c(
    person("Aron", "Atkins", , "aron@posit.co", role = c("aut", "cre")),
    person("Toph", "Allen", role = "aut"),
    person("Kevin", "Ushey", role = "aut"),
    person("Jonathan", "McPherson", role = "aut"),
    person("Joe", "Cheng", role = "aut"),
    person("JJ", "Allaire", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Manage the R packages your project depends on in an isolated,
    portable, and reproducible way.
License: GPL-2
URL: https://github.com/rstudio/packrat
BugReports: https://github.com/rstudio/packrat/issues
Depends: R (>= 3.0.0)
Imports: tools, utils
Suggests: devtools, httr, knitr, mockery, rmarkdown, testthat (>=
        3.0.0), webfakes, withr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-06-16 19:37:19 UTC; aron
Author: Aron Atkins [aut, cre],
  Toph Allen [aut],
  Kevin Ushey [aut],
  Jonathan McPherson [aut],
  Joe Cheng [aut],
  JJ Allaire [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Aron Atkins <aron@posit.co>
Repository: CRAN
Date/Publication: 2025-06-16 20:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 17:41:47 UTC; unix
