Package: packrat
Type: Package
Version: 0.0.0
Repository: CRAN
License: GPL
Description: Packrat test package
Title: Packrat test package
Author: Anonymous Person <Anonymous@rstudio.org>
Maintainer: Anonymous Person <Anonymous@rstudio.org>


