# Generated by roxygen2: do not edit by hand

export(.snapshotImpl)
export(bundle)
export(bundles_dir)
export(clean)
export(disable)
export(extlib)
export(get_lockfile_metadata)
export(get_opts)
export(init)
export(install)
export(install_local)
export(lib_dir)
export(off)
export(on)
export(opts)
export(packify)
export(packrat_lib)
export(packrat_mode)
export(project_dir)
export(repos_add)
export(repos_add_local)
export(repos_create)
export(repos_list)
export(repos_remove)
export(repos_set)
export(repos_set_local)
export(repos_upload)
export(restore)
export(search_path)
export(set_lockfile_metadata)
export(set_opts)
export(snapshot)
export(src_dir)
export(status)
export(unbundle)
export(unused_packages)
export(user_lib)
export(with_extlib)
import(utils)
importFrom(tools,md5sum)
importFrom(tools,pkgVignettes)
