% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search-path.R
\name{search_path}
\alias{search_path}
\title{Get Packages on the Search Path}
\usage{
search_path()
}
\description{
Retrieve the packages on the search path, as well as the
associated library location.
}
